package com.king.wechat.qrcode.scanning.analyze;

import android.graphics.Bitmap;

import com.king.mlkit.vision.camera.AnalyzeResult;
import com.king.mlkit.vision.camera.analyze.Analyzer;
import com.king.mlkit.vision.camera.util.BitmapUtils;
import com.king.mlkit.vision.camera.util.LogUtils;
import com.king.wechat.qrcode.WeChatQRCodeDetector;

import org.opencv.core.Mat;

import java.util.ArrayList;
import java.util.List;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.camera.core.ImageProxy;

/**
 * @author <a href="mailto:jenly1314@gmail.com">Jenly</a>
 */
public class WeChatScanningAnalyzer implements Analyzer<List<String>> {

    /**
     * 是否需要返回二维码的各个顶点
     */
    private boolean isNeedVertices;

    public WeChatScanningAnalyzer(){
        this(false);
    }

    /**
     * 构造
     * @param isNeedVertices 是否需要返回二维码的各个顶点
     */
    public WeChatScanningAnalyzer(boolean isNeedVertices){
        this.isNeedVertices = isNeedVertices;
    }

    @Override
    public void analyze(@NonNull ImageProxy imageProxy, @NonNull Analyzer.OnAnalyzeListener<AnalyzeResult<List<String>>> listener) {
        try {
            final Bitmap bitmap = BitmapUtils.getBitmap(imageProxy);
            AnalyzeResult<List<String>> result = detectAndDecode(bitmap, isNeedVertices);
            if(result != null){
                listener.onSuccess(result);
            }else{
                listener.onFailure();
            }
        } catch (Exception e) {
            LogUtils.w(e);
        }
    }

    /**
     * 检测并识别二维码
     * @param bitmap
     * @param isNeedVertices
     * @return
     */
    @Nullable
    private AnalyzeResult<List<String>> detectAndDecode(Bitmap bitmap,boolean isNeedVertices) {
        if(isNeedVertices){//如果需要返回二维码的各个顶点
            final List<Mat> points = new ArrayList<>();
            List<String> result = WeChatQRCodeDetector.detectAndDecode(bitmap, points);
            if(result != null && result.size() > 0){
                return new QRCodeAnalyzeResult<>(bitmap, result, points);
            }
        }else{//反之则需识别结果即可
            List<String> result = WeChatQRCodeDetector.detectAndDecode(bitmap);
            if(result != null && result.size() > 0){
                return new QRCodeAnalyzeResult<>(bitmap, result);
            }
        }

        return null;
    }

    public static class QRCodeAnalyzeResult<T> extends AnalyzeResult<T> {

        private List<Mat> points;

        public QRCodeAnalyzeResult() {
            super();
        }

        public QRCodeAnalyzeResult(Bitmap bitmap, T result) {
            super(bitmap,result);
        }

        public QRCodeAnalyzeResult(Bitmap bitmap, T result,List<Mat> points) {
            super(bitmap,result);
            this.points = points;
        }

        public List<Mat> getPoints() {
            return points;
        }

        public void setPoints(List<Mat> points) {
            this.points = points;
        }
    }
}
